/**************************************************************************//**
 * @file     main.c
 * @version  V1.00
 * $Revision: 9 $
 * $Date: 15/09/02 10:04a $
 * @brief
 *           Change duty cycle and period of output waveform by PWM Double Buffer function.
 *
 * @note
 * Copyright (C) 2014~2015 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "M451Series.h"

/*---------------------------------------------------------------------------------------------------------*/
/* Macro, type and constant definitions                                                                    */
/*---------------------------------------------------------------------------------------------------------*/
#define PLL_CLOCK       144000000

/*---------------------------------------------------------------------------------------------------------*/
/* Global variables                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/

/**
 * @brief       PWM0 IRQ Handler
 *
 * @param       None
 *
 * @return      None
 *
 * @details     ISR to handle PWM0 interrupt event
 */
void PWM0P0_IRQHandler(void)
{
    static int toggle = 0;

    // Update PWM0 channel 0 period and duty
    toggle++;
    if(toggle >= 60000)
    {
    	toggle = 0;
#ifdef DUTY_STAGGER    	
    	ChangeDuty(PWM0, 0, 500, 2, 1500, 4, 1000);
#else    	
    	ChangeDuty(PWM0, 0, 500, 2, 500, 0, 0);
#endif    	
    }
    // Clear channel 0 period interrupt flag
    PWM_ClearPeriodIntFlag(PWM0, 0);
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Enable HIRC clock (Internal RC 22.1184MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRCEN_Msk);

    /* Waiting for HIRC clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV0_HCLK(1));

    /* Enable HXT clock (external XTAL 12MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Waiting for HXT clock ready */
    CLK_WaitClockReady(CLK_STATUS_HXTSTB_Msk);

    /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK);

    /* Waiting for PLL clock ready */
    CLK_WaitClockReady(CLK_STATUS_PLLSTB_Msk);

    /* Enable PWM0 module clock */
    CLK_EnableModuleClock(PWM0_MODULE);

    /*---------------------------------------------------------------------------------------------------------*/
    /* PWM clock frequency configuration                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Select HCLK clock source as PLL and and HCLK clock divider as 2 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_PLL, CLK_CLKDIV0_HCLK(2));

    /* PWM clock frequency can be set equal or double to HCLK by choosing case 1 or case 2 */
    /* case 1.PWM clock frequency is set equal to HCLK: select PWM module clock source as PCLK */
    CLK_SetModuleClock(PWM0_MODULE, CLK_CLKSEL2_PWM0SEL_PCLK0, NULL);

    /* case 2.PWM clock frequency is set double to HCLK: select PWM module clock source as PLL */
    //CLK_SetModuleClock(PWM0_MODULE, CLK_CLKSEL2_PWM0SEL_PLL, NULL);
    /*---------------------------------------------------------------------------------------------------------*/

    /* Enable UART module clock */
    CLK_EnableModuleClock(UART0_MODULE);

    /* Select UART module clock source as HXT and UART module clock divider as 1 */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UARTSEL_HXT, CLK_CLKDIV0_UART(1));

    /* Reset PWM0 module */
    SYS_ResetModule(PWM0_RST);

    /* Update System Core Clock */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set PD multi-function pins for UART0 RXD and TXD */
    SYS->GPD_MFPL &= ~(SYS_GPD_MFPL_PD0MFP_Msk | SYS_GPD_MFPL_PD1MFP_Msk);
    SYS->GPD_MFPL |= (SYS_GPD_MFPL_PD0MFP_UART0_RXD | SYS_GPD_MFPL_PD1MFP_UART0_TXD);

    /* Set PC multi-function pins for PWM0 Channel0~5 */
    SYS->GPC_MFPL = (SYS->GPC_MFPL & (~SYS_GPC_MFPL_PC0MFP_Msk));
    SYS->GPC_MFPL |= SYS_GPC_MFPL_PC0MFP_PWM0_CH0;
    SYS->GPC_MFPL = (SYS->GPC_MFPL & (~SYS_GPC_MFPL_PC1MFP_Msk));
    SYS->GPC_MFPL |= SYS_GPC_MFPL_PC1MFP_PWM0_CH1;
    SYS->GPC_MFPL = (SYS->GPC_MFPL & (~SYS_GPC_MFPL_PC2MFP_Msk));
    SYS->GPC_MFPL |= SYS_GPC_MFPL_PC2MFP_PWM0_CH2;
    SYS->GPC_MFPL = (SYS->GPC_MFPL & (~SYS_GPC_MFPL_PC3MFP_Msk));
    SYS->GPC_MFPL |= SYS_GPC_MFPL_PC3MFP_PWM0_CH3;
	SYS->GPC_MFPL = (SYS->GPC_MFPL & (~SYS_GPC_MFPL_PC4MFP_Msk));
    SYS->GPC_MFPL |= SYS_GPC_MFPL_PC4MFP_PWM0_CH4;
	SYS->GPC_MFPL = (SYS->GPC_MFPL & (~SYS_GPC_MFPL_PC5MFP_Msk));
    SYS->GPC_MFPL |= SYS_GPC_MFPL_PC5MFP_PWM0_CH5;
}

void UART0_Init()
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART module */
    SYS_ResetModule(UART0_RST);

    /* Configure UART0 and set UART0 baud rate */
    UART_Open(UART0, 115200);
}


/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main(void)
{
    /* Init System, IP clock and multi-function I/O
       In the end of SYS_Init() will issue SYS_LockReg()
       to lock protected register. If user want to write
       protected register, please issue SYS_UnlockReg()
       to unlock protected register if necessary */

    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Init System, IP clock and multi-function I/O */
    SYS_Init();

    /* Lock protected registers */
    SYS_LockReg();

    /* Init UART to 115200-8n1 for print message */
    UART0_Init();

	/*clock source 72M, period 20K*/
	PWM_SET_PRESCALER(PWM0, 0, 0);
	PWM_SET_PRESCALER(PWM0, 2, 0);
	PWM_SET_PRESCALER(PWM0, 4, 0);
	
	PWM0->CTL1 = ((PWM_UP_COUNTER<<0) | (PWM_UP_COUNTER<<2) | (PWM_UP_COUNTER<<4));//up counter
    PWM0->CTL1 &= ~(PWM_CTL1_CNTMODE0_Msk | PWM_CTL1_CNTMODE2_Msk | PWM_CTL1_CNTMODE4_Msk);// set PWM to auto-reload mode
    PWM0->CTL1 |= (PWM_CTL1_OUTMODE0_Msk | PWM_CTL1_OUTMODE2_Msk | PWM_CTL1_OUTMODE4_Msk);//complementary mode

    PWM_SET_CNR(PWM0, 0, 3599);//72M/3600 = 20K
    PWM_SET_CNR(PWM0, 2, 3599);//72M/3600 = 20K
    PWM_SET_CNR(PWM0, 4, 3599);//72M/3600 = 20K
#ifdef DUTY_STAGGER	
    ChangeDuty(PWM0, 0, 1000, 2, 1000, 4, 1000);
#else    
    ChangeDuty(PWM0, 0, 1000, 2, 1000, 0, 0);
#endif    
	
	PWM_EnablePeriodInt(PWM0, 0, 0);
    NVIC_EnableIRQ(PWM0P0_IRQn);
	// Enable output of PWM0 channel 0
    PWM_EnableOutput(PWM0, PWM_CH_0_MASK|PWM_CH_2_MASK|PWM_CH_4_MASK);

    // Start
    PWM_Start(PWM0, PWM_CH_0_MASK|PWM_CH_2_MASK|PWM_CH_4_MASK);

    while(1);

}


#ifdef DUTY_STAGGER
int ChangeDuty(PWM_T *pwm, uint32_t u32ChannelNum0, uint32_t duty0, 
				uint32_t u32ChannelNum2, uint32_t duty2,
				uint32_t u32ChannelNum4, uint32_t duty4)
{
	
	/*ch0*/	
	pwm->CMPDAT[u32ChannelNum0] = duty0;
	pwm->CMPDAT[u32ChannelNum0+1] = pwm->CMPDAT[u32ChannelNum0];	

	/* zero high and period do nothing */
	PWM0->WGCTL0 = (PWM0->WGCTL0 & ~((PWM_WGCTL0_PRDPCTL0_Msk | PWM_WGCTL0_ZPCTL0_Msk) << (u32ChannelNum0 * 2))) 
						| (PWM_OUTPUT_HIGH << (u32ChannelNum0 * 2));	
	/* ch0 up toggle and down do noting */
	PWM0->WGCTL1 = (PWM0->WGCTL1 & ~((PWM_WGCTL1_CMPDCTL0_Msk | PWM_WGCTL1_CMPUCTL0_Msk) << (u32ChannelNum0 * 2)))
		                | (PWM_OUTPUT_TOGGLE << (u32ChannelNum0 * 2));    

	/*ch2*/
	pwm->CMPDAT[u32ChannelNum2] = pwm->CMPDAT[u32ChannelNum0];
	pwm->CMPDAT[u32ChannelNum2+1] = (duty2+pwm->CMPDAT[u32ChannelNum2]);
	/* zero low and center do nothing */
	PWM0->WGCTL0 = (PWM0->WGCTL0 & ~((PWM_WGCTL0_PRDPCTL0_Msk | PWM_WGCTL0_ZPCTL0_Msk) << (u32ChannelNum2 * 2))) 
						| (PWM_OUTPUT_LOW << (u32ChannelNum2 * 2));	
	/* ch2 up toggle */
	PWM0->WGCTL1 = (PWM0->WGCTL1 & ~((PWM_WGCTL1_CMPDCTL0_Msk | PWM_WGCTL1_CMPUCTL0_Msk) << (u32ChannelNum2 * 2)))
		                | (PWM_OUTPUT_TOGGLE << (u32ChannelNum2 * 2));	
	/* ch3 up toggle */
	PWM0->WGCTL1 = (PWM0->WGCTL1 & ~((PWM_WGCTL1_CMPDCTL0_Msk | PWM_WGCTL1_CMPUCTL0_Msk) << ((u32ChannelNum2+1) * 2)))
		                | (PWM_OUTPUT_TOGGLE << ((u32ChannelNum2+1) * 2));
	
	/*ch4*/
	pwm->CMPDAT[u32ChannelNum4] = pwm->CMPDAT[u32ChannelNum2+1];
	pwm->CMPDAT[u32ChannelNum4+1] = pwm->CMPDAT[u32ChannelNum4] + duty4;
	/* zero low and center do nothing */
	PWM0->WGCTL0 = (PWM0->WGCTL0 & ~((PWM_WGCTL0_PRDPCTL0_Msk | PWM_WGCTL0_ZPCTL0_Msk) << (u32ChannelNum4 * 2))) 
						| (PWM_OUTPUT_LOW << (u32ChannelNum4 * 2));	
	/* ch4 up toggle */
	PWM0->WGCTL1 = (PWM0->WGCTL1 & ~((PWM_WGCTL1_CMPDCTL0_Msk | PWM_WGCTL1_CMPUCTL0_Msk) << (u32ChannelNum4 * 2)))
		                | (PWM_OUTPUT_TOGGLE << (u32ChannelNum4 * 2));
	/* ch5 up toggle */
	PWM0->WGCTL1 = (PWM0->WGCTL1 & ~((PWM_WGCTL1_CMPDCTL0_Msk | PWM_WGCTL1_CMPUCTL0_Msk) << ((u32ChannelNum4+1) * 2)))
		                | (PWM_OUTPUT_TOGGLE << ((u32ChannelNum4+1) * 2));
}
#else
/* phase 180 degree  */
int ChangeDuty(PWM_T *pwm, uint32_t u32ChannelNum0, uint32_t duty0, 
				uint32_t u32ChannelNum2, uint32_t duty2,
				uint32_t u32ChannelNum4, uint32_t duty4)
{
	
	/*ch0*/	
	pwm->CMPDAT[u32ChannelNum0] = duty0;
	pwm->CMPDAT[u32ChannelNum0+1] = pwm->CMPDAT[u32ChannelNum0];	

	/* zero high and period do nothing */
	PWM0->WGCTL0 = (PWM0->WGCTL0 & ~((PWM_WGCTL0_PRDPCTL0_Msk | PWM_WGCTL0_ZPCTL0_Msk) << (u32ChannelNum0 * 2))) 
						| (PWM_OUTPUT_HIGH << (u32ChannelNum0 * 2));	
	/* ch0 up toggle and down do noting */
	PWM0->WGCTL1 = (PWM0->WGCTL1 & ~((PWM_WGCTL1_CMPDCTL0_Msk | PWM_WGCTL1_CMPUCTL0_Msk) << (u32ChannelNum0 * 2)))
		                | (PWM_OUTPUT_TOGGLE << (u32ChannelNum0 * 2));    

	/*ch2*/
	pwm->CMPDAT[u32ChannelNum2] = pwm->PERIOD[u32ChannelNum0]/2;
	pwm->CMPDAT[u32ChannelNum2+1] = (duty2+pwm->CMPDAT[u32ChannelNum2]);
	/* zero low and center do nothing */
	PWM0->WGCTL0 = (PWM0->WGCTL0 & ~((PWM_WGCTL0_PRDPCTL0_Msk | PWM_WGCTL0_ZPCTL0_Msk) << (u32ChannelNum2 * 2))) 
						| (PWM_OUTPUT_LOW << (u32ChannelNum2 * 2));	
	/* ch2 up toggle */
	PWM0->WGCTL1 = (PWM0->WGCTL1 & ~((PWM_WGCTL1_CMPDCTL0_Msk | PWM_WGCTL1_CMPUCTL0_Msk) << (u32ChannelNum2 * 2)))
		                | (PWM_OUTPUT_TOGGLE << (u32ChannelNum2 * 2));	
	/* ch3 up toggle */
	PWM0->WGCTL1 = (PWM0->WGCTL1 & ~((PWM_WGCTL1_CMPDCTL0_Msk | PWM_WGCTL1_CMPUCTL0_Msk) << ((u32ChannelNum2+1) * 2)))
		                | (PWM_OUTPUT_TOGGLE << ((u32ChannelNum2+1) * 2));		
}

#endif